import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  ClassType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  Injectable,
} from '@overflow/commons/di/decorators';

export interface ConfigurationAnnotationAttributes {
  name?: string[];
}

@Injectable()
export class ConfigurationAnnotation implements Annotation {
  public readonly attributes: ConfigurationAnnotationAttributes;

  public constructor(name: string | ConfigurationAnnotationAttributes) {

    if (undefined === name) {
      this.attributes = {
      };
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes = {
        name: [<string>name],
      };
    } else {
      this.attributes = <ConfigurationAnnotationAttributes>name;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    // no op
  }

}

export const Configuration = Decorator.create(ConfigurationAnnotation);
